//-----------------------------------------------------------------------------
//file : TriPhoto.xaml.cs
//brief : Principal Window
//author : Benjamin Maillard
//date : 31 Mars 2009
//-----------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Forms;
using System.IO;


namespace TriPhoto
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class TriPhotoApp : System.Windows.Window
    {
        #region Constructeur
        public TriPhotoApp()
        {
            InitializeComponent();

            ImageElementManager.Manager.ListImageNonClasser = ImageListStack;
            ImageElementManager.Manager.ListImageADeterminer = ImageListADeterminer;
            ImageElementManager.Manager.ListImageASupprimer = ImageListASupprimer;
            ImageElementManager.Manager.ListImageAGarder = ImageListAGarder;
            ImageElementManager.Manager.PrincipaleView = ImageStack;
            ImageElementManager.Manager.PopupView = ImagePopupCanvas;
            ImageElementManager.Manager.LoadProgress += new ImageElementManager.LoadProgressEvent(LoadProgress);
            ImageElementManager.Manager.LoadBegin += new ImageElementManager.LoadBeginEvent(Manager_LoadBegin);
            ImageElementManager.Manager.LoadEnd += new ImageElementManager.LoadEndEvent(Manager_LoadEnd);

            ImageListBorderASupprimer.Drop += new System.Windows.DragEventHandler(ImageListBorderASupprimer_Drop);
            ImageListBorderADeterminer.Drop += new System.Windows.DragEventHandler(ImageListBorderADeterminer_Drop);
            ImageListBorderAGarder.Drop += new System.Windows.DragEventHandler(ImageListBorderAGarder_Drop);
            ImageStackBorder.Drop += new System.Windows.DragEventHandler(ImageStackBorder_Drop);
            ImageListStackBorder.Drop += new System.Windows.DragEventHandler(ImageListStackBorder_Drop);

            AjoutRepertoire(TriPhoto.Properties.Resources.IKeepTooltip);

            ImageListScrollViewer.KeyUp += new System.Windows.Input.KeyEventHandler(ImageListScrollViewer_KeyUp);

            //Raccouci clavier
            this.KeyUp += new System.Windows.Input.KeyEventHandler(TriPhotoApp_KeyUp);
        }


       

 
        #endregion

        #region membres
        private string m_filePaths = "";
        #endregion

        #region callback

        void ImageListScrollViewer_KeyUp(object sender, System.Windows.Input.KeyEventArgs e)
        {
            List<ImageElement> l_selectedImage = null;
            int l_selectIndex = 0;

            l_selectedImage = ImageElementManager.Manager.GetSelectedImage();
            if (l_selectedImage.Count > 0)
                l_selectIndex = ImageListStack.Children.IndexOf(l_selectedImage[0].Border);

            switch (e.Key)
            {
                case Key.Down:
                    if(ImageListStack.Children.Count > l_selectIndex + 1)
                        ImageElementManager.Manager.Select((ImageListStack.Children[l_selectIndex + 1] as Border).Tag as ImageElement);
                    break;
                case Key.Up:
                    if ( l_selectIndex - 1 >= 0)
                        ImageElementManager.Manager.Select((ImageListStack.Children[l_selectIndex - 1] as Border).Tag as ImageElement);

                    break;
            }
        }

        /// <summary>
        /// Gestion des raccoucis clavier
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void TriPhotoApp_KeyUp(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == Key.L)
            {
                this.RotateUnClockWise(this,null);
            }


            if (e.Key == Key.R)
            {
                this.RotateClockWise(this, null);
            }
        }

        #region Bouton de la barre de commande
        /// <summary>
        /// Choix du repertoire contenant les images
        /// </summary>
        public void ChooseDirectory(object sender, RoutedEventArgs e)
        {
            FolderBrowserDialog l_chooseDialog = new FolderBrowserDialog();

            if (l_chooseDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                m_filePaths = l_chooseDialog.SelectedPath;
                ImageElementManager.Manager.AnalyseDirectory(m_filePaths);
            }
        }

        /// <summary>
        /// Quitter
        /// </summary>
        public void ExitApplication(object sender, RoutedEventArgs e)
        {
            if (System.Windows.MessageBox.Show(TriPhoto.Properties.Resources.ExitQuestion, TriPhoto.Properties.Resources.QuitTooltip, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                Close();
        }

        /// <summary>
        /// Change l'espace couleur des images selectionnes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void ColorChange(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_list = ImageElementManager.Manager.GetSelectedImage();
            foreach (ImageElement l_image in l_list)
            { 
                l_image.ChangeColorMode();
            }
        }

        /// <summary>
        /// Tourne les images selectionnes de 90
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void RotateClockWise(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_list = ImageElementManager.Manager.GetSelectedImage();
            foreach (ImageElement l_image in l_list)
            {
                l_image.RotateClockWise();
            }
        }

        /// <summary>
        /// Tourne les images selectionnes de 90
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void RotateUnClockWise(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_list = ImageElementManager.Manager.GetSelectedImage();
            foreach (ImageElement l_image in l_list)
            {
                l_image.RotateUnClockWise();
            }
        }

        /// <summary>
        /// Clique sur mode comparer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void CompareMode(object sender, RoutedEventArgs e)
        {
            ImageElementManager.Manager.CompareMode = (bool)CompareButton.IsChecked;
        }
        #endregion

        #region Chargement
        /// <summary>
        /// Progression du chargement des images
        /// </summary>
        /// <param name="in_pourcent"></param>
        /// <param name="in_filePath"></param>
        public void LoadProgress(int in_pourcent, string in_filePath)
        {
            LoadProgressBar.Value = in_pourcent;
            LoadFileName.Content = in_filePath;

            //On laisse du temps
            System.Windows.Forms.Application.DoEvents();
        }
        /// <summary>
        /// Fin de chargement des images
        /// </summary>
        void Manager_LoadEnd()
        {
            LoadProgressBar.Visibility = Visibility.Hidden;
            LoadFileName.Visibility = Visibility.Hidden;
        }
        /// <summary>
        /// Debut de chargement des images
        /// </summary>
        void Manager_LoadBegin()
        {
            LoadProgressBar.Visibility = Visibility.Visible;
            LoadFileName.Visibility = Visibility.Visible;
        }
        #endregion

        #region Drag and Drop
        /// <summary>
        /// Drop dans la liste a supprimer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageListBorderASupprimer_Drop(object sender, System.Windows.DragEventArgs e)
        {
            ChangeStateOfImageList(e.Data.GetData(System.Windows.DataFormats.Locale) as List<ImageElement>,
                                   ImageState.A_SUPPRIMER);
        }

        /// <summary>
        /// Drop dans la liste des images en entre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageListStackBorder_Drop(object sender, System.Windows.DragEventArgs e)
        {
            ChangeStateOfImageList(e.Data.GetData(System.Windows.DataFormats.Locale) as List<ImageElement>,
                                   ImageState.NON_CLASSE);
        }

        /// <summary>
        /// Drop dans l'espace de visualisation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageStackBorder_Drop(object sender, System.Windows.DragEventArgs e)
        {
            if (sender == ImageStackBorder)
                return;

            List<ImageElement> l_imageList = e.Data.GetData(System.Windows.DataFormats.Locale) as List<ImageElement>;
            if (l_imageList.Count == 1)
            {
                if(e.Source != l_imageList[0])
                    ImageElementManager.Manager.Show(l_imageList[0]);
            }
            else
            {
                if (l_imageList.Count > 1)
                    ImageElementManager.Manager.CompareMode = true;
                
                foreach (ImageElement l_image in l_imageList)
                {
                    ImageElementManager.Manager.Show(l_image);
                }
            }
        }

        /// <summary>
        /// Drop dans l'esapce des image a garder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageListBorderAGarder_Drop(object sender, System.Windows.DragEventArgs e)
        {
            ChangeStateOfImageList(e.Data.GetData(System.Windows.DataFormats.Locale) as List<ImageElement>,
                                   ImageState.A_GARDER);
        }

        /// <summary>
        /// Drop dans l'espace des images non determine
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ImageListBorderADeterminer_Drop(object sender, System.Windows.DragEventArgs e)
        {
            ChangeStateOfImageList(e.Data.GetData(System.Windows.DataFormats.Locale) as List<ImageElement>,
                                   ImageState.NON_DETERMINE);
        }

        /// <summary>
        /// Change l'etat d'une liste d'imageElement
        /// </summary>
        /// <param name="in_imageList"></param>
        /// <param name="in_state"></param>
        private void ChangeStateOfImageList(List<ImageElement> in_imageList, ImageState in_state)
        {
            //Changement d'etat de toute les images
            foreach (ImageElement l_image in in_imageList)
                l_image.ImageState = in_state;
        }

        #endregion

        #region Boutton images de passage d'une liste a une autre
        /// <summary>
        /// Clique sur je garde
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void JeGarde(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_selectedImage = ImageElementManager.Manager.GetSelectedImage();
            
            int l_selectIndex = 0;
            if(l_selectedImage.Count > 0)
                l_selectIndex = ImageListStack.Children.IndexOf(l_selectedImage[0].Border);
            
            foreach (ImageElement l_image in l_selectedImage)
            {
                l_image.ImageState =  ImageState.A_GARDER;
            }

            //Si il n'y a qu'un element de visualis on passe au suivant
            if (l_selectIndex != -1 && ImageStack.Children.Count == 1 && ImageListStack.Children.Count > 0)
            {
                ImageElementManager.Manager.Select((ImageListStack.Children[l_selectIndex] as Border).Tag as ImageElement);
            }
            else
            {
                ImageElementManager.Manager.UnSelectAll();
            }
        }

        /// <summary>
        /// Clique sur je ne veux plus garder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void JeGardePlus(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_selectedImage = ImageElementManager.Manager.GetSelectElementOfChildren(ImageListAGarder.Children);

            foreach (ImageElement l_image in l_selectedImage)
            {
                l_image.ImageState = ImageState.NON_CLASSE;
            }

            //Si pas d'image selectionn on enleve la dernire de la liste
            if (l_selectedImage.Count == 0 && ImageListAGarder.Children.Count > 0)
            {
                ((ImageListAGarder.Children[ImageListAGarder.Children.Count - 1] as Border).Tag as ImageElement).ImageState = ImageState.NON_CLASSE;
            }
        }

        /// <summary>
        /// Clique sur je supprime
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void JeSupprime(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_selectedImage = ImageElementManager.Manager.GetSelectedImage();
            int l_selectIndex = 0;
            if (l_selectedImage.Count > 0)
                l_selectIndex = ImageListStack.Children.IndexOf(l_selectedImage[0].Border);


            foreach (ImageElement l_image in l_selectedImage)
            {
                l_image.ImageState = ImageState.A_SUPPRIMER;
            }

            //Si il n'y a qu'un element de visualis on passe au suivant
            if (l_selectIndex != -1 && ImageStack.Children.Count == 1 && ImageListStack.Children.Count > 0)
            {
                ImageElementManager.Manager.Select((ImageListStack.Children[l_selectIndex] as Border).Tag as ImageElement);
            }
            else
            {
                ImageElementManager.Manager.UnSelectAll();
            }
        }

        /// <summary>
        /// Clique sur je ne veux plus supprime
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void JeSupprimePlus(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_selectedImage = ImageElementManager.Manager.GetSelectElementOfChildren(ImageListASupprimer.Children);

            foreach (ImageElement l_image in l_selectedImage)
            {
                l_image.ImageState = ImageState.NON_CLASSE;
            }

            //Si pas d'image selectionn on enleve la dernire de la liste
            if (l_selectedImage.Count == 0 && ImageListASupprimer.Children.Count > 0)
            {
                ((ImageListASupprimer.Children[ImageListASupprimer.Children.Count - 1] as Border).Tag as ImageElement).ImageState = ImageState.NON_CLASSE;
            }
        }

        /// <summary>
        /// Clique sur je ne sais pas
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void JeSaisPas(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_selectedImage = ImageElementManager.Manager.GetSelectedImage();

            int l_selectIndex = 0;
            if (l_selectedImage.Count > 0)
                l_selectIndex = ImageListStack.Children.IndexOf(l_selectedImage[0].Border);

            foreach (ImageElement l_image in l_selectedImage)
            {
                l_image.ImageState = ImageState.NON_DETERMINE;
            }

            //Si il n'y a qu'un element de visualis on passe au suivant
            if (l_selectIndex != -1 && ImageStack.Children.Count == 1 && ImageListStack.Children.Count > 0)
            {
                ImageElementManager.Manager.Select((ImageListStack.Children[l_selectIndex] as Border).Tag as ImageElement);
            }
            else
            {
                ImageElementManager.Manager.UnSelectAll();
            }
        }

        /// <summary>
        /// Clique sur je ne sais vraiment pas
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void JeSaisPasPlus(object sender, RoutedEventArgs e)
        {
            List<ImageElement> l_selectedImage = ImageElementManager.Manager.GetSelectElementOfChildren(ImageListADeterminer.Children);

            foreach (ImageElement l_image in l_selectedImage)
            {
                l_image.ImageState = ImageState.NON_CLASSE;
            }

            //Si pas d'image selectionn on enleve la dernire de la liste
            if (l_selectedImage.Count == 0 && ImageListADeterminer.Children.Count > 0)
            {
                ((ImageListADeterminer.Children[ImageListADeterminer.Children.Count - 1] as Border).Tag as ImageElement).ImageState = ImageState.NON_CLASSE;
            }
        }
        #endregion

        #region Bouton de gestion des repertoires
        /// <summary>
        /// Ajoute un repertoire 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void AjoutRepertoire(object sender, RoutedEventArgs e)
        {
            SaisieRepertoire l_saisieRep = new SaisieRepertoire();

            if (l_saisieRep.ShowDialog() == false)
            {
                return;
            }

            AjoutRepertoire(l_saisieRep.NomRepertoire);
        }

        /// <summary>
        /// Ajoute un repertoire
        /// </summary>
        /// <param name="in_nomRep"></param>
        private void AjoutRepertoire(string in_nomRep)
        {
            ImageElementManager.Manager.AddDirectory(in_nomRep);

            ImageElementManager.Manager.SetDefaultDirectory(in_nomRep);

            CreateBoutonNewDirectory(in_nomRep);
        }

        /// <summary>
        /// Creation du bouton de nouveau repertoire
        /// </summary>
        /// <param name="in_nomRep"></param>
        private void CreateBoutonNewDirectory(string in_nomRep)
        {
            System.Windows.Controls.Button l_button = new System.Windows.Controls.Button();
            l_button.Background = Brushes.WhiteSmoke;
            int l_lenght = in_nomRep.Length < 8 ? in_nomRep.Length : 8;
            l_button.Content = in_nomRep.Substring(0, l_lenght);
            l_button.FontSize = 18;
            l_button.FontWeight = FontWeights.UltraBold;
            l_button.Click += new RoutedEventHandler(SetRepertoireCourrant);
            l_button.Height = 50;
            l_button.Tag = in_nomRep;
            l_button.ToolTip = in_nomRep;

            l_button.ContextMenu = new System.Windows.Controls.ContextMenu();
            l_button.ContextMenu.Tag = l_button;

            System.Windows.Controls.MenuItem l_renItemMenu = new System.Windows.Controls.MenuItem();
            l_renItemMenu.Header = "Renommer";
            l_renItemMenu.Tag = in_nomRep;
            l_renItemMenu.Click += new RoutedEventHandler(RenItemMenu_Click);
            l_button.ContextMenu.Items.Add(l_renItemMenu);

            System.Windows.Controls.MenuItem l_suppItemMenu = new System.Windows.Controls.MenuItem();
            l_suppItemMenu.Header = "Supprimer";
            l_suppItemMenu.Tag = in_nomRep;
            l_suppItemMenu.Click += new RoutedEventHandler(SuppItemMenu_Click);
            l_button.ContextMenu.Items.Add(l_suppItemMenu);

            menuWrapPanel.Children.Add(l_button);

            Title = "TriPhoto (" + in_nomRep + ")";
            JeGardeLabel.Content = in_nomRep;
        }

        /// <summary>
        /// Clique sur le menu supprimer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 
        void SuppItemMenu_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Controls.MenuItem l_menu = sender as System.Windows.Controls.MenuItem;

            ImageElementManager.Manager.RemDirectoy(l_menu.Tag as string);

            System.Windows.Controls.ContextMenu l_context = l_menu.Parent as System.Windows.Controls.ContextMenu;

            menuWrapPanel.Children.Remove(l_context.Tag as System.Windows.Controls.Button);
        }

        /// <summary>
        /// Clique sur le menu supprimer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void RenItemMenu_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Controls.MenuItem l_menu = sender as System.Windows.Controls.MenuItem;

            SaisieRepertoire l_saisieRep = new SaisieRepertoire();

            if (l_saisieRep.ShowDialog() == false)
            {
                return;
            }

            ImageElementManager.Manager.RenomerDirectory(l_menu.Tag as string, l_saisieRep.NomRepertoire);

            //On met a jour les menus contextuel
            l_menu.Tag = l_saisieRep.NomRepertoire;
            System.Windows.Controls.ContextMenu l_menuContext = l_menu.Parent as System.Windows.Controls.ContextMenu;

            //On met a jour les autres menu contextuel
            foreach (System.Windows.Controls.MenuItem l_menuItem in l_menuContext.Items)
            {
                if (l_menuItem == l_menu)
                    continue;
                l_menuItem.Tag = l_saisieRep.NomRepertoire;
            }
            
            //Mise a jour du button
            System.Windows.Controls.Button l_button = l_menuContext.Tag as System.Windows.Controls.Button;
            int l_lenght = l_saisieRep.NomRepertoire.Length < 8 ? l_saisieRep.NomRepertoire.Length : 8;
            l_button.Content = l_saisieRep.NomRepertoire.Substring(0, l_lenght);
            l_button.Tag = l_saisieRep.NomRepertoire;
            l_button.ToolTip = l_saisieRep.NomRepertoire;
        }

        /// <summary>
        /// Set le repertoire courrant
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void SetRepertoireCourrant(object sender, RoutedEventArgs e)
        {
            System.Windows.Controls.Button l_button =  sender as System.Windows.Controls.Button ;
            ImageElementManager.Manager.SetDefaultDirectory(l_button.Tag as string);
            Title = "TriPhoto (" + l_button.Tag + ")";
            JeGardeLabel.Content = l_button.Tag;
        }

        /// <summary>
        /// Tourne les images selectionnes de 90
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Appliquer(object sender, RoutedEventArgs e)
        {
            MessageBoxResult l_MessageBoxResult = System.Windows.MessageBox.Show(TriPhoto.Properties.Resources.SaveRotationAndBlackWhite, TriPhoto.Properties.Resources.SaveOption, MessageBoxButton.YesNoCancel, MessageBoxImage.Question, MessageBoxResult.Yes);

            switch (l_MessageBoxResult)
            { 
                case MessageBoxResult.Yes:
                    ImageElementManager.Manager.Transfert(true);
                    break;
                case MessageBoxResult.No:
                    ImageElementManager.Manager.Transfert(false);
                    break;
            }
           
        }

        /// <summary>
        /// Ouvre un projet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Open(object sender, RoutedEventArgs e)
        {
            OpenFileDialog l_openFile = new OpenFileDialog();
            l_openFile.Filter = "prj files (*.prj)|*.prj";
            l_openFile.RestoreDirectory = true;

            List<string> l_newDirectoryList = new List<string>();

            //Rechargement d'un projet
            if (l_openFile.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                ImageElementManager.Manager.LoadProject(l_openFile.FileName, ref l_newDirectoryList);
            }


            if (l_newDirectoryList.Count >= 1)
            {

                //Creation des boutons
                foreach (string l_newDirectory in l_newDirectoryList)
                {
                    CreateBoutonNewDirectory(l_newDirectory);
                }

                Title = "TriPhoto (" + l_newDirectoryList[l_newDirectoryList.Count - 1] + ")";
                JeGardeLabel.Content = l_newDirectoryList[l_newDirectoryList.Count - 1];
            }

        }

        /// <summary>
        ///  Nouveau projet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void New(object sender, RoutedEventArgs e)
        {
            if (System.Windows.Forms.MessageBox.Show(TriPhoto.Properties.Resources.CreateNew, TriPhoto.Properties.Resources.NewProject, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                ImageElementManager.Manager.Clear();
        }

        /// <summary>
        /// Sauvegarde un projet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void Save(object sender, RoutedEventArgs e)
        {
            SaveFileDialog l_saveFile = new SaveFileDialog();
            l_saveFile.Filter = "prj files (*.prj)|*.prj";
            l_saveFile.RestoreDirectory = true;

            if (l_saveFile.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                ImageElementManager.Manager.SaveProject(l_saveFile.FileName);
            }
        }
        #endregion

        #endregion

        #region methodes


        #endregion
    }
}